module.exports = function() {

  const Queue = require('./queue');
  const EventEmitter = require('events');
  const spawn = require('child_process').spawn;

  class Sync extends EventEmitter {

    constructor(opts) {
      super();
      this.opts = opts;
      this.queue = new Queue();
    }

    _sync(additionalArgs=[]) {
      let start = new Date();
      let { source, target, exclude } = this.opts;
      let excludes = exclude.map(name => `--exclude="${name}"`);
      let args = ['rsync', '-avvz', '--delete', ...additionalArgs, ...excludes, `${source}/`, `${target}/` ];
      return new Promise((resolve, reject) => {
        let child = spawn('/bin/sh', [ '-c', args.join(' ') ], {
          detached: false
        });

        let log = (stream, enabled) => data => {
          if(!enabled) {
            return;
          }
          if(data instanceof Buffer) {
            data = data.toString('utf8');
          }
          process[stream].write(data);
        };

        let done = code => {
          let end = new Date();
          if(code > 0) {
            reject(new Error(`sync spawn failed: ${code}`));
          } else {
            this.emit('done', end - start);
            resolve();
          }
        };

        child.stderr.on('data', log('stderr', true));
        child.stdout.on('data', log('stdout', false));

        child.on('exit', done);
        child.on('error', err => reject(err));
      });
    }

    all() {
      return this.queue.add(() => this._sync());
    }

    _subset(files) {
      let includes = [];
      files.map(path => {
        let components = path.split('/');
        for(let i = 0; i < components.length; i++) {
          includes.push(components.slice(0, i + 1).join('/'));
        }
      });
      let include = includes.map(file => `--include="${file}"`);
      let args = [ ...include, '--exclude="*"' ];
      return this.queue.add(() => this._sync(args));
    }

    subset(files) {
      if(files.length > 25) {
        return this.all();
      }
      return this._subset(files);
    }

    start() {
      return this.all();
    }

  }

  return Sync;
}();
